#include <iostream>
#include <sstream>
#include <iterator>
#include "bot.h"

using namespace std;
using namespace __gnu_cxx;

Bot::Bot(string name, string type){
	_name = name;
	_type = type;
}

void Bot::play(){
	cout << "reg: " << _name << " " << _type << endl;

	string input;
	getline(cin, input);
	if (input != "wsk\\")
		return;
	getline(cin, input);
	_name = value<string>(input);
	cerr << "Got name: " << _name << endl;

	//robber and 5 cops
	getline(cin, input);
	_players[value<string>(input)].type = "robber";
	getline(cin, input);
	_players[value<string>(input)].type = "cop-foot";
	getline(cin, input);
	_players[value<string>(input)].type = "cop-foot";
	getline(cin, input);
	_players[value<string>(input)].type = "cop-foot";
	getline(cin, input);
	_players[value<string>(input)].type = "cop-foot";

	cerr << "Got players, building graph." << endl;
	getline(cin, input);
	buildGraph();
	getline(cin, input);

	while (true){
		getline(cin, input);
		if (input == "game-over")
			return;
		updateWorld();
		_type = _players[_name].type;
		_location = _players[_name].location;
		getline(cin, input);
		move(turn());
	}
}

Bot::~Bot(){
}

/**
 	  	 	  	nod\ eol	
 	  	 	  	( nod: loc node-tag coordinate coordinate eol )*	
 	  	 	  	nod/ eol	
 	  	 	  	edg\ eol	
 	  	 	  	( edg: loc loc edge-type eol )*	
 	  	 	  	edg/ eol	
*/
void Bot::buildGraph(){
	string input;
	getline(cin, input);
	if (input != "nod\\")
		return;

	cerr << "Getting intersections" << endl;
	while (true){
		getline(cin, input);
		if (input == "nod/")
			break;
		istringstream node(input);
		node >> input;
		node >> input;
		Intersection& inter = _intersections[input];
		node >> inter.type;
		node >> inter.x;
		node >> inter.y;
	}

	cerr << "Number of intersections: " << _intersections.size() << endl;

	getline(cin, input);
	if (input != "edg\\")
		return;

	cerr << "Getting streets" << endl;
	int streets = 0;
	while (true){
		getline(cin, input);
		if (input == "edg/")
			break;
		++streets;
		istringstream street(input);
		string from;
		street >> from;
		string to;
		street >> to;
		string type;
		street >> type;
		if (type == "foot"){
			_intersections[from].connections[to] = both;
			Intersection& inter = _intersections[to];
			if (inter.connections.find(from) == inter.connections.end())
				inter.connections[from] = foot;
		}else
			_intersections[from].connections[to] = car;
	}
	cerr << "Number of streets: " << streets << endl;
}

void Bot::updateWorld(){
	string input;
	getline(cin,input);
	_world = value<int>(input);
	cerr << "World: " << _world << endl;

	getline(cin,input);
	_robbed = value<int>(input);
	cerr << "Robbed: " << _robbed << endl;

	getline(cin,input);
	while (true){
		getline(cin, input);
		if (input == "bv/")
			break;
		istringstream bank(input);
		bank >> input;
		bank >> input;
		bank >> _banks[input];
	}
	cerr << "Number of banks: " << _banks.size() << endl;
	
	getline(cin,input);
	while (true){
		getline(cin, input);
		if (input == "ev/")
			break;
		istringstream evidence(input);
	}
	
	getline(cin,input);
	_smell = value<int>(input);

	getline(cin,input);
	while (true){
		getline(cin, input);
		if (input == "pl/")
			break;
		istringstream player(input);
		player >> input;
		player >> input;
		cerr << "Player: " << input << endl;
		Player& pl = _players[input];
		player >> pl.location;
		player >> pl.type;
	}
	cerr << "Number of players: " << _players.size() << endl;
}

void Bot::move(std::string location){
	cout << "mov: " << location << " " << _type << endl;
}

template<class T>
T value(std::string input){
	istringstream istr(input);
	istr >> input;
	T s;
	istr >> s;
	return s;
}

std::string Bot::turn(){
	return _location;
}

